import unittest

from pyarxaas.hierarchy import DateHierarchyBuilder


class DateHierarchyBuildTest(unittest.TestCase):

    def test_init(self):
        DateHierarchyBuilder(
            "yyyy-MM-dd HH:mm:SSS",
            DateHierarchyBuilder.Granularity.SECOND_MINUTE_HOUR_DAY_MONTH_YEAR,
            DateHierarchyBuilder.Granularity.MINUTE_HOUR_DAY_MONTH_YEAR)

    def test__request_payload(self):
        expected = {
            "builder" : {
                "type" : "dateBased",
                "dateFormat" : "yyyy-MM-dd HH:mm",
                "granularities" : [ "SECOND_MINUTE_HOUR_DAY_MONTH_YEAR", "MINUTE_HOUR_DAY_MONTH_YEAR", "HOUR_DAY_MONTH_YEAR", "DAY_MONTH_YEAR" ]
            }
        }
        db = DateHierarchyBuilder(
            "yyyy-MM-dd HH:mm",
            DateHierarchyBuilder.Granularity.SECOND_MINUTE_HOUR_DAY_MONTH_YEAR,
            DateHierarchyBuilder.Granularity.MINUTE_HOUR_DAY_MONTH_YEAR,
            DateHierarchyBuilder.Granularity.HOUR_DAY_MONTH_YEAR,
            DateHierarchyBuilder.Granularity.DAY_MONTH_YEAR)
        self.assertEqual(expected, db._request_payload())


from enum import Enum
from typing import Mapping


class DateHierarchyBuilder:
    """ Understands building hierarchies for date values"""

    class Granularity(Enum):
        SECOND_MINUTE_HOUR_DAY_MONTH_YEAR = "smhdmy"
        MINUTE_HOUR_DAY_MONTH_YEAR = "mhdmy"
        HOUR_DAY_MONTH_YEAR = "hdmy"
        DAY_MONTH_YEAR = "dmy"
        WEEK_MONTH_YEAR = "wmy"
        WEEK_YEAR = "wy"
        MONTH_YEAR = "my"
        WEEKDAY = "wd"
        WEEK = "w"
        QUARTER = "q"
        YEAR = "y"
        DECADE = "d"
        CENTURY = "c"
        MILLENIUM = "m"

    def __init__(self, date_format, *granularities):
        self._date_format = date_format
        self._granularities = granularities

    def _request_payload(self) -> Mapping:
        return {
            "builder": {
                "type": "dateBased",
                "dateFormat": self._date_format,
                "granularities": [granularity.name for granularity  in self._granularities]
            }
        }




from pyarxaas.hierarchy.interval_builder.interval_hierarchy_builder import IntervalHierarchyBuilder
from pyarxaas.hierarchy.redaction_hierarchy_builder import RedactionHierarchyBuilder
from pyarxaas.hierarchy.order_builder.order_hierarchy_builder import OrderHierarchyBuilder
from pyarxaas.hierarchy.date_hierarchy_builder import DateHierarchyBuilder

__all__ = ['IntervalHierarchyBuilder', 'RedactionHierarchyBuilder', 'OrderHierarchyBuilder', 'DateHierarchyBuilder']